#!/bin/zsh

##========================================================================================
##                                                                                      ##
##                                       Variáveis                                      ##
##                                                                                      ##
##========================================================================================

# 🇬🇧 Commands history
# 🇵🇹 Histórico de comandos
HIST_STAMPS="dd/mm/yyyy"
HISTFILE=$HOME/.zshrc_history
HISTSIZE=2000
SAVEHIST=2000
export HIST_STAMPS="dd/mm/yyyy"
export HISTFILE=$HOME/.zshrc_history
export HISTSIZE=2000
export SAVEHIST=2000

# 🇬🇧 ZSH settings
# 🇵🇹 Configurações do ZSH
CASE_SENSITIVE="true"
COMPLETION_WAITING_DOTS="true"
DISABLE_AUTO_UPDATE="true"
DISABLE_UPDATE_PROMPT="true"
ENABLE_CORRECTION="false"
HYPHEN_INSENSITIVE="true"
export CASE_SENSITIVE="true"
export COMPLETION_WAITING_DOTS="true"
export DISABLE_AUTO_UPDATE="true"
export DISABLE_UPDATE_PROMPT="true"
export ENABLE_CORRECTION="false"
export HYPHEN_INSENSITIVE="true"

# 🇬🇧 User-defined common ENV variables 
# 🇵🇹 Variáveis ENV comuns definidos pelo utilizador

# Editor
PREFERRED_EDITOR="micro"

case $PREFERRED_EDITOR in
"nvim")
  export EDITOR="nvim"
  ;;
"vim")
  export EDITOR="vim"
  ;;
"micro")
  export EDITOR="micro"
  ;;
*)
  export EDITOR="nano"
  ;;
esac

# 🇬🇧 Less, for example, to view git log page
# 🇵🇹 Less, por exmeplo, visualizar a página de git log
PREFERRED_PAGER="grc less"

case $PREFERRED_PAGER in
"nvimpage")
  export PAGER="nvimpager -p"
  ;;
"grc less")
  export PAGER="grc less"
  ;;
*)
  export PAGER="less"
  ;;
esac

export COLORTERM="truecolor"
export SHELL="$(which zsh)"
export TERM="xterm-256color"

export LANG="pt_PT.utf8"
export LANGUAGE="pt_PT:pt_BR:en"
export LC_ALL="pt_PT.UTF-8"

vscode=$(command -v code)

if [[ -x "$vscode" ]]; then
    export VISUAL="code"
else
    export VISUAL="code-insiders"
fi

##========================================================================================
##                                                                                      ##
##                         Exportações de pastas de executáveis                         ##
##                                                                                      ##
##========================================================================================

#.NET
if [ -d $HOME/.dotnet ]; then
    export PATH="$HOME/.dotnet:$PATH"
fi

# Cargo
if [ -d "$HOME/.cargo/bin" ]; then
  export PATH="$HOME/.cargo/bin:$PATH"
fi

# Go
if [ -d "$HOME/.go" ]; then
  export GOPATH=~/.go
fi

# MikTeX
if [ -d /opt/miktex ]; then
    export PATH="/opt/miktex/bin:$PATH"
fi

export MIKTEX_USERCONFIG=$HOME/.miktex/texmfs/config
export MIKTEX_USERDATA=$HOME/.miktex/texmfs/data
export MIKTEX_USERINSTALL=$HOME/.miktex/texmfs/install

# NPM
if [ -d "$HOME/node_modules/.bin" ]; then
  export PATH="$HOME/node_modules/.bin:$PATH"
else
  export PATH="$HOME/.node_modules/.bin:$PATH"
fi

# NVM
if [ -d "$HOME/.nvm" ]; then
  export NVM_DIR="$HOME/.nvm"
  [ -s "$NVM_DIR/nvm.sh" ] && \. "$NVM_DIR/nvm.sh"
  [ -s "$NVM_DIR/bash_completion" ] && \. "$NVM_DIR/bash_completion"
fi

# Ruby
if [ -d "$HOME/.rbenv/bin" ]; then
  export PATH="$HOME/.rbenv/bin:$PATH"
  export PATH="$HOME/.rbenv/plugins/ruby-build/bin:$PATH"
  eval "$(rbenv init -)"

  # 🇬🇧 Indexing Colorls
  # 🇵🇹 Indexação do Colorls
  if [ -x $(command -v colorls) ]; then
    source $(dirname $(gem which colorls))/tab_complete.sh
  else
    echo "Inexistente" &>/dev/null;
  fi
fi

# Yarn
if [ -d $HOME/.yarn/bin ]; then
    export PATH="$HOME/.yarn/bin:$PATH"
fi

# Main local executables
# Executáveis locais principais
if [[ $OSTYPE = (darwin|freebsd)* ]]; then
    [ -d "/opt/local/bin" ] && export PATH="/opt/local/bin:$PATH"
fi

if [[ $OSTYPE = (linux)* ]]; then
    if [ -d "$HOME/.local/bin" ]; then
      export PATH="$HOME/.local/bin:$PATH"
    fi

    # Linuxbrew (Homebrew)
    if [ -d "/home/linuxbrew/.linuxbrew/bin" ]; then
      eval $(/home/linuxbrew/.linuxbrew/bin/brew shellenv)
    fi

    if [ -d "/usr/local/bin" ]; then
      export PATH="/usr/local/sbin:$PATH"
    fi
fi

# ls colours options variables
# Variáveis de opções de cores de ls
if [[ $OSTYPE = (darwin|freebsd)* ]]; then
    export CLICOLOR=true
    export LSCOLORS='exfxcxdxbxGxDxabagacad'
fi

if [[ $OSTYPE = (linux)* ]]; then
    export LS_OPTIONS='--color=auto'
fi

# FZF
## 🇬🇧 Searching the plugins and sourcing them
## 🇵🇹 A localizar os complementos e obtê-los
[ -f $HOME/.asdf/asdf.sh ] && . $HOME/.asdf/asdf.sh
[ -f $HOME/.asdf/completions/asdf.bash ] && . $HOME/.asdf/completions/asdf.bash
[ -f $HOME/.fzf.zsh ] && source $HOME/.fzf.zsh

## 🇬🇧 FZF user-redefined settings
## 🇵🇹 Configurações do FZF redefinidas pelo utilizador
export FZF_ALT_C_COMMAND='rg --hidden --files --null --sort path 2>/dev/null | xargs -0 dirname | uniq'
export FZF_COMPLETION_TRIGGER='**'
export FZF_CTRL_T_COMMAND="$FZF_DEFAULT_COMMAND"
export FZF_CTRL_T_OPTS='--preview "/usr/bin/cat {} | nvimpager -c "'
export FZF_DEFAULT_COMMAND='rg --files --hidden 2>/dev/null'
export FZF_DEFAULT_OPTS='-m --bind ctrl-a:select-all,ctrl-d:deselect-all,ctrl-t:toggle-all'

# 🇬🇧 Using lesspipe with less
# 🇵🇹 Utilizar lesspipe com less
lesspipe=$(command -v lesspipe.sh)

if [[ -x $lesspipe ]]; then
  export LESSOPEN="|lesspipe.sh %s"
fi

thf=$(command -v thefuck)

if [[ -x $thf ]]; then
  eval $(thefuck --alias)
fi

drc=$(command -v dircolors)

if [[ -x $drc ]]; then
  eval `dircolors ~/.dircolors`
fi